// 002 u_c_pascal_to_html
// 24 dec 2001

// -- (C) John COLIBRI 2001
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_pascal_to_html;
  interface
    uses u_c_text_file;

    type c_pascal_to_html= class(c_text_file)
                             public
                               Constructor create_pascal_to_html(p_name, p_file_name: String); Virtual;
                               procedure transform_to_html;
                               Destructor Destroy; Override;
                           end;

  implementation
    uses SysUtils, u_loaded, u_types_constants, u_c_display;

    procedure start;
      // for debug: loading address
      begin
      end; // start

    // -- c_pascal_to_html

    Constructor c_pascal_to_html.create_pascal_to_html(p_name, p_file_name: String);
      begin
        Inherited create_text_file('pascal', p_file_name);
      end; // create_pascal_to_html

    procedure c_pascal_to_html.transform_to_html;
      const k_special_characters= ['<', '>'];
      var l_special_array: array[' '..#255] of String[10];

      procedure initialize_special_characters;
        begin
          FillChar(l_special_array, SizeOf(l_special_array), 0);
          l_special_array['<']:= '&lt;';
          l_special_array['>']:= '&gt;';
        end; // initialize_special_characters

      // -- output handling
      var l_output: File;
          l_end_of_last_flush: Integer;

      procedure flush_string(p_string: String);
        begin
          if Length(p_string)> 0
            then begin
                BlockWrite(l_output, p_string[1], Length(p_string));
              end;
          l_end_of_last_flush:= Self.m_buffer_index;
        end; // flush_string

      procedure flush_line(p_string: String);
        begin
          flush_string(p_string+ k_return+ k_line_feed);
        end; // flush_line)

      var l_start_index: Integer;

      procedure flush_previous;
        var l_pending: String;
            l_pending_start, l_pending_index: Integer;
        begin
          // -- check for <, > etc
          l_pending_start:= l_end_of_last_flush;
          l_pending_index:= l_end_of_last_flush;

          while (l_pending_index< m_buffer_index) and (m_buffer_index< m_buffer_size) do
            if m_pt_buffer[l_pending_index] in k_special_characters
              then begin
                  flush_string(f_extract_string_start_end(l_pending_start, l_pending_index- 1));
                  flush_string(l_special_array[m_pt_buffer[l_pending_index]]);
                  Inc(l_pending_index);
                  l_pending_start:= l_pending_index;
               end
             else Inc(l_pending_index);

          flush_string(f_extract_string_start_end(l_pending_start, l_pending_index- 1));

          // -- next chunk
          l_start_index:= m_buffer_index;
        end; // flush_previous

      procedure create_file;
        var l_output_name: String;
        begin
          with m_c_file_name do
            l_output_name:= f_drive_and_path+ m_name+ '.html';
          display(l_output_name);

          Assign(l_output, l_output_name);
          Rewrite(l_output, 1);
          
          flush_line('<HTML>');
          flush_line('  <BODY>');
        end; // create_file

      procedure close_output_file;
        begin
          flush_line('');
          flush_line('  </BODY>');
          flush_line('</HTML>');
          Close(l_output);
        end; // close_output_file

      procedure handle_the_file;

        const k_comment_color= 'FF00FF'; // RgB = Pruple
              k_keyword_color= 'FF0000'; // Rgb = red
              k_string_litteral_color= '0000FF'; // rgB = Blue
              k_identifier_color= '0'; // black

              k_reserved_max= 69;

              k_reserved_array: array[0..k_reserved_max] of STRING =
                ('AND','ARRAY','AS','ASM','BEGIN',
                 'CASE', 'CLASS', 'CONST', 'CONST', 'CONSTRUCTOR',
                 'DESTRUCTOR',  'DISPINTERFACE', 'DIV', 'DO', 'DOWNTO',
                 'ELSE', 'END', 'EXCEPT', 'EXPORTS', 'FILE',
                 'FINALIZATION', 'FINALLY', 'FOR', 'FUNCTION', 'GOTO',
                 'IF', 'IMPLEMENTATION', 'IN', 'INHERITED', 'INITIALIZATION',
                 'INLINE', 'INTERFACE', 'IS', 'LABEL', 'LIBRARY',
                 'MOD', 'NIL', 'NOT', 'OBJECT', 'OF',
                 'OR', 'OUT', 'PACKED', 'PRIVATE', 'PROCEDURE', 'PROGRAM',
                 'PROPERTY', 'PUBLIC','RAISE', 'RECORD', 'REPEAT', // 'RESOURCESTRING',
                 'SET', 'SHL', 'SHR', 'STRING', // 'STRINGRESOURCE',
                 'THEN', 'THREADVAR', 'TO', 'TRY', 'TYPE',
                 'UNIT', 'UNTIL', 'USES', 'VAR', 'WHILE',
                 'WITH', 'XOR',
                 'OVERRIDE', 'VIRTUAL', 'sentinell');

        function f_color(p_color_value: String): String ;
          begin
            Result:= '<FONT COLOR="'+ p_color_value+ '">';
          end; // f_color

        procedure handle_identifiers;

          function f_is_reserved(p_identifier: String): Boolean;
            var l_index: Integer;
            begin
              // place the sentinell
              k_reserved_array[k_reserved_max]:= p_identifier;
              l_index:= 0;
              while k_reserved_array[l_index]<> p_identifier do
                Inc(l_index);

              Result:= l_index< k_reserved_max;
            end; // f_is_resereved

          const k_identifier_characters= ['a'..'z', 'A'..'Z', '_']+ ['0'..'9'];
          var l_identifier: String;

          begin
            flush_previous;

            while (m_buffer_index< m_buffer_size)
                and (m_pt_buffer[m_buffer_index] in k_identifier_characters) do
              Inc(m_buffer_index);

            l_identifier:= f_extract_string_start_end(l_start_index, m_buffer_index- 1);
            if f_is_reserved(UpperCase(l_identifier))
               then flush_string(f_color(k_keyword_color)+ '<B>'+ l_identifier+ '</B></FONT>')
              else flush_string(f_color(k_identifier_color)+ '<I>'+ l_identifier+ '</I></FONT>');
          end; // handle_identifiers

        var l_has_sent_return: Boolean;

        procedure handle_return;
          begin
            flush_previous;

            // -- 2 or more consecutive returns => a blank line (<P>)
            if l_has_sent_return
              then flush_string('<P>')
              else flush_string('<BR>');

            Inc(m_buffer_index, 2);
            l_has_sent_return:= true;
          end; // handle_return

        procedure handle_spaces;
          var l_spaces: String;
          begin
            flush_previous;

            // -- build a string to avoid multiple flushes
            l_spaces:= '';
            while (m_buffer_index< m_buffer_size) and  (m_pt_buffer[m_buffer_index]= ' ') do
            begin
              l_spaces:= l_spaces+ '&nbsp;';
              Inc(m_buffer_index);
            end;

            flush_string(l_spaces);
          end; // handle_spaces

        procedure handle_double_slash_comments;
          begin
            flush_previous;

            // -- everything until Return
            flush_string(f_color(k_comment_color));

            while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index]<> k_return) do
              if m_pt_buffer[m_buffer_index]= ' '
                then handle_spaces
                else Inc(m_buffer_index);

            flush_previous;
            flush_string('</FONT>');
          end; // handle_double_slash_comments

        procedure handle_bracket_comment;
          begin
            flush_previous;

            flush_string(f_color(k_comment_color));
            while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index]<> '}') do
              if m_pt_buffer[m_buffer_index]= k_return
                then handle_return
                else begin
                    l_has_sent_return:= False;
                    if m_pt_buffer[m_buffer_index]= ' '
                      then handle_spaces
                      else Inc(m_buffer_index);
                  end;

            // -- the ending bracket
            Inc(m_buffer_index);

            flush_previous;
            flush_string('</FONT>');
          end; // handle_bracket_comment

        procedure handle_parenthesis_slash_comments;
          begin
            flush_previous;

            flush_string(f_color(k_comment_color));
            while (m_buffer_index+ 1< m_buffer_size)
                and not ((m_pt_buffer[m_buffer_index]= '*')
                          and (m_pt_buffer[m_buffer_index+ 1]= ')')) do
              if m_pt_buffer[m_buffer_index]= k_return
                then handle_return
                else begin
                    l_has_sent_return:= False;
                    if m_pt_buffer[m_buffer_index]= ' '
                      then handle_spaces
                      else Inc(m_buffer_index);
                  end;

            // -- add "*)"
            Inc(m_buffer_index, 2);

            flush_previous;
            flush_string('</FONT>');
          end; // handle_parenthesis_slash_comments

        procedure handle_string_litteral;
          begin
            flush_previous;

            flush_string(f_color(k_string_litteral_color));
            // -- skip the first '
            Inc(m_buffer_index);

            repeat
              if m_pt_buffer[m_buffer_index]= ' '
                then handle_spaces
                else
                  if (m_pt_buffer[m_buffer_index]<> '''')
                    then inc(m_buffer_index)
                    else
                      if m_pt_buffer[m_buffer_index+ 1]= ''''
                        then begin
                            inc(m_buffer_index, 2);
                          end
                        else break;
            until m_buffer_index>= m_buffer_size;

            // -- the last quote
            Inc(m_buffer_index);
            flush_previous;
            flush_string('</FONT>');
          end; // handle_string_litteral

        begin // handle_the_file
          l_has_sent_return:= False;

          while m_buffer_index< m_buffer_size do
          begin
            case m_pt_buffer[m_buffer_index] of
              'a'..'z', 'A'..'Z', '_' : handle_identifiers;
              k_return : handle_return;
              '/' : if (m_buffer_index+ 1< m_buffer_size) and (m_pt_buffer[m_buffer_index+ 1]= '/')
                      then handle_double_slash_comments
                      else Inc(m_buffer_index);
              '{' : handle_bracket_comment;
              '(' : if (m_buffer_index+ 1< m_buffer_size) and (m_pt_buffer[m_buffer_index+ 1]= '*')
                      then handle_parenthesis_slash_comments
                      else Inc(m_buffer_index);
              ' ': handle_spaces;
              '''' : handle_string_litteral;
              else Inc(m_buffer_index);
            end; // case

            if (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index]<> k_return)
              then l_has_sent_return:= False;
          end;
        end; // handle_the_file

      begin // transform_to_html
        if f_load
          then begin
              create_file;
              initialize_special_characters;

              m_buffer_index:= 0;

              handle_the_file;

              close_output_file;
              close_file;
            end; // f_load
      end; // transform_to_html

    Destructor c_pascal_to_html.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_pascal_to_html
      display_loaded('u_c_pascal_to_html', @start);
    end. // u_c_pascal_to_html
